"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const parse_wanted_dependency_1 = __importDefault(require("@pnpm/parse-wanted-dependency"));
const semver = require("semver");
function default_1(overrides) {
    const genericVersionOverrides = [];
    const versionOverrides = [];
    Object.entries(overrides)
        .forEach(([selector, newPref]) => {
        if (selector.includes('>')) {
            const [parentSelector, childSelector] = selector.split('>');
            versionOverrides.push({
                newPref,
                parentWantedDependency: parse_wanted_dependency_1.default(parentSelector),
                wantedDependency: parse_wanted_dependency_1.default(childSelector),
            });
            return;
        }
        genericVersionOverrides.push({
            newPref,
            wantedDependency: parse_wanted_dependency_1.default(selector),
        });
    });
    return ((pkg) => {
        overrideDepsOfPkg(pkg, versionOverrides.filter(({ parentWantedDependency }) => {
            return parentWantedDependency.alias === pkg.name && (!parentWantedDependency.pref || semver.satisfies(pkg.version, parentWantedDependency.pref));
        }));
        overrideDepsOfPkg(pkg, genericVersionOverrides);
        return pkg;
    });
}
exports.default = default_1;
function overrideDepsOfPkg(pkg, versionOverrides) {
    if (pkg.dependencies)
        overrideDeps(versionOverrides, pkg.dependencies);
    if (pkg.optionalDependencies)
        overrideDeps(versionOverrides, pkg.optionalDependencies);
    return pkg;
}
function overrideDeps(versionOverrides, deps) {
    for (const versionOverride of versionOverrides) {
        if (deps[versionOverride.wantedDependency.alias] &&
            (!versionOverride.wantedDependency.pref ||
                deps[versionOverride.wantedDependency.alias] === versionOverride.wantedDependency.pref)) {
            deps[versionOverride.wantedDependency.alias] = versionOverride.newPref;
        }
    }
}
//# sourceMappingURL=createVersionsOverrider.js.map